/*
	threshold regression example
	written for Timberlake Consultants
	(timberlake.co.uk)
	by Robert Grant of BayesCamp
	(bayescamp.com)
*/




// get data

clear all
use "kenley.dta", clear
tsset datetime, clocktime delta(1 hour)
generate decimalday = day + (hour/24)
generate hoursine = sin((hour-11)/12*3.14)

line temperature decimalday, ///
	xtick(1(1)30, grid) xlabel(5(5)25) xtitle("Day in August 2018") ///
	graphregion(color(white)) ytitle("Temperature (Celsius)") ///
 name(temp, replace)
line humidity decimalday, ///
	xtick(1(1)30, grid) xlabel(5(5)25) xtitle("Day in August 2018") ///
	graphregion(color(white)) ytitle("Humidity (%)") ///
	name(humid, replace)
line press decimalday, ///
	xtick(1(1)30, grid) xlabel(5(5)25) xtitle("Day in August 2018") ///
	graphregion(color(white)) ytitle("Surface pressure change (mmHg)") ///
	name(press, replace)
graph combine temp humid press, cols(1) graphregion(color(white)) ///
	name(threeseries, replace)
graph export "threeseries.png", replace




// regression on hoursine only

regress temperature hoursine
predict temp_sine

twoway (line temperature decimalday, lcolor(navy)) ///
	(line temp_sine decimalday, lcolor(dkorange) lpattern(dash)), ///
	xtick(1(1)30, grid) xlabel(5(5)25) xtitle("Day in August 2018") ///
	graphregion(color(white)) ytitle("Temperature (Celsius)") ///
	legend(order(1 "Observed" 2 "Predicted")) name(tempsine, replace)
graph export "temperature_sine.png", replace

 
threshold temperature, threshvar(decimalday) regionvars(hoursine)

predict temp_sine_thresh1

matrix TH1 = e(thresholds)
global TH1n = TH1[1,2]

twoway (line temperature decimalday, lcolor(navy)) ///
	(line temp_sine_thresh1 decimalday, lcolor(dkorange) lpattern(dash)), ///
	xtick(1(1)30, grid) xlabel(5(5)25) xtitle("Day in August 2018") ///
	graphregion(color(white)) ytitle("Temperature (Celsius)") ///
	legend(order(1 "Observed" 2 "Predicted")) name(tempsine1, replace)
graph export "temperature_sine_thresh1.png", replace




// more thresholds

threshold temperature, threshvar(decimalday) regionvars(hoursine) optthresh(4)

predict temp_sine_thresh4
twoway (line temperature decimalday, lcolor(navy)) ///
	(line temp_sine_thresh4 decimalday, lcolor(dkorange) lpattern(dash)), ///
	xtick(1(1)30, grid) xlabel(5(5)25) xtitle("Day in August 2018") ///
	graphregion(color(white)) ytitle("Temperature (Celsius)") ///
	legend(order(1 "Observed" 2 "Predicted")) name(tempsine4, replace)
graph export "temperature_sine_thresh4.png", replace

matrix TH4 = e(thresholds)
global TH4n = e(nthresholds)
global TH4n1 = $TH4n - 1
forvalues i=1/$TH4n {
	local THindex = `i'+1
	global TH4_`i' = TH4[1,`THindex']
}

line temperature decimalday, ///
	xtick(1(1)30, grid) xlabel(5(5)25) xtitle("Day in August 2018") ///
	graphregion(color(white)) ytitle("Temperature (Celsius)") ///
	xline(${TH4_1}, lcolor(purple)) xline(${TH4_2}, lcolor(purple)) ///
	xline(${TH4_3}, lcolor(purple)) xline(${TH4_4}, lcolor(purple)) ///
	xline(${TH1n}, lcolor(dkorange)) name(temp_thresh, replace)
graph export "temp_thresholds.png", replace




// regression on more predictors

regress temperature L.humid L.press hoursine
threshold temperature, threshvar(decimalday) regionvars(L.humid L.press hoursine)

matrix TH1 = e(thresholds)
global TH1n = TH1[1,2]

predict temp_thresh1
twoway (line temperature decimalday, lcolor(navy)) ///
	(line temp_thresh1 decimalday, ///
		lcolor(dkorange) lpattern(dash) xline(${TH1n}, lcolor(dkorange))), ///
	xtick(1(1)30, grid) xlabel(5(5)25) xtitle("Day in August 2018") ///
	graphregion(color(white)) ytitle("Temperature (Celsius)") ///
	legend(order(1 "Observed" 2 "Predicted")) name(temppred1, replace)
graph export "temperature_thresh1.png", replace

threshold temperature, threshvar(decimalday) ///
	regionvars(L.humid L.press hoursine) optthresh(4)

predict temp_thresh4
twoway (line temperature decimalday, lcolor(navy)) ///
	(line temp_thresh4 decimalday, lcolor(dkorange) lpattern(dash) ///
		xline(${TH4_1}, lcolor(purple)) xline(${TH4_2}, lcolor(purple)) ///
		xline(${TH4_3}, lcolor(purple)) xline(${TH4_4}, lcolor(purple)) ///
		xline(${TH1n}, lcolor(dkorange))), ///
	xtick(1(1)30, grid) xlabel(5(5)25) xtitle("Day in August 2018") ///
	graphregion(color(white)) ytitle("Temperature (Celsius)") ///
	legend(order(1 "Observed" 2 "Predicted")) name(temppred4, replace)
graph export "temperature_thresh4.png", replace

matrix TH4 = e(thresholds)
global TH4n = e(nthresholds)
global TH4n1 = $TH4n - 1
forvalues i=1/$TH4n {
	local THindex = `i'+1
	global TH4_`i' = TH4[1,`THindex']
}

line temperature decimalday, ///
	xtick(1(1)30, grid) xlabel(5(5)25) xtitle("Day in August 2018") ///
	graphregion(color(white)) ytitle("Temperature (Celsius)") ///
	xline(${TH4_1}, lcolor(purple)) xline(${TH4_2}, lcolor(purple)) ///
	xline(${TH4_3}, lcolor(purple)) xline(${TH4_4}, lcolor(purple)) ///
	xline(${TH1n}, lcolor(dkorange)) name(temp_thr, replace)
line humidity decimalday, ///
	xtick(1(1)30, grid) xlabel(5(5)25) xtitle("Day in August 2018") ///
	graphregion(color(white)) ytitle("Humidity (%)") ///
	xline(${TH4_1}, lcolor(purple)) xline(${TH4_2}, lcolor(purple)) ///
	xline(${TH4_3}, lcolor(purple)) xline(${TH4_4}, lcolor(purple)) ///
	xline(${TH1n}, lcolor(dkorange)) name(humid_thr, replace)
line press decimalday, ///
	xtick(1(1)30, grid) xlabel(5(5)25) xtitle("Day in August 2018") ///
	graphregion(color(white)) ytitle("Surface pressure change (mmHg)") ///
	xline(${TH4_1}, lcolor(purple)) xline(${TH4_2}, lcolor(purple)) ///
	xline(${TH4_3}, lcolor(purple)) xline(${TH4_4}, lcolor(purple)) ///
	xline(${TH1n}, lcolor(dkorange)) name(press_thr, replace)
graph combine temp_thr humid_thr press_thr, cols(1) graphregion(color(white)) ///
	name(threeseries_thresholds, replace)
graph export "threeseries_thresholds.png", replace

regress temperature L.humid L.press hoursine if decimalday <= 23.667
regress temperature L.humid L.press hoursine if decimalday > 23.667
regress temperature L.press hoursine if decimalday > 23.667


